/*****************************
 * ECE 4760: Final Project   *
 * Human Tetris              *
 * Video Arcade Edition      *
 *                           *
 * Adam Papamarcos (aip23)   *
 * Kerran Flanagan (kaf42)   *
 * Cornell University        *
 * Spring 2010               *
 *****************************/

/****************************************************************************
 *   Copyright (C) 2009 by Claas Anders "CaScAdE" Rathje                    *
 *   admiralcascade@gmail.com                                               *
 *   Project-URL: http://www.mylifesucks.de/oss/c-osd/                      *
 *                                                                          *
 *   This program is free software; you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by   *
 *   the Free Software Foundation; either version 2 of the License.         *
 *                                                                          *
 *   This program is distributed in the hope that it will be useful,        *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for more details.                           *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to the                          *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 *                                                                          *
 *   Extended and modified for ATmega644 in 2010 by                         *
 *   Kerran Flanagan and Adam Papamarcos.                                   *
 ****************************************************************************/

#include <util/delay.h>
#include <avr/pgmspace.h>
#include "macros.h"
#include "max7456_learn.h"
#include "max7456_spi.h"

#if WRITECHARS
#include "max7456_characters.h"
// ========== FUNCTION DEFINITIONS ==========


/**
 * let the MAX7456 learn a new character at <number>
 * with <data>.
 */
void learn_char(uint8_t number, unsigned char* data) {
	// select character to write (CMAH)
	spi_send_byte(0x09, number);

	for (uint8_t i = 0; i < 54; i++) {
		// select 4pixel byte of char (CMAL)
		spi_send_byte(0x0A, i);

		// write 4pixel byte of char (CMDI)
		spi_send_byte(0x0B, data[i]);
	}

	// write to the NVM array from the shadow RAM (CMM)
	spi_send_byte(0x08, 0b10100000);

	// according to maxim writing to nvram takes about 12ms, lets wait longer
	_delay_ms(24);
}

/**
 * let the MAX7456 learn a new character at <number>
 * with <data> from progmem
 */
void learn_char_from_progmem(uint8_t number, const char* data) {
	// select character to write (CMAH)
	spi_send_byte(0x09, number);

	for (uint8_t i = 0; i < 54; i++) {
		// select 4pixel byte of char (CMAL)
		spi_send_byte(0x0A, i);

		// write 4pixel byte of char (CMDI)
		spi_send_byte(0x0B, pgm_read_byte(&data[i]));
	}

	// write to the NVM array from the shadow RAM (CMM)
	spi_send_byte(0x08, 0b10100000);

	// according to maxim writing to nvram takes about 12ms, lets wait longer
	_delay_ms(120);

}

// IMPORTANT: LEARNING MORE THAN 8 CHARACTERS PER ON/OFF CYCLE CAUSES ERRORS
void learnCharacters(uint8_t whatChars) {

	MAX_RESET_LOW
	_delay_ms(400); //hold it low to reset
	MAX_RESET_HIGH
	_delay_ms(100); //wait for it to stabilize again

	// 8bit mode
	spi_send_byte(0x04, 0b01000000);
	// DISABLE display (VM0)
	spi_send_byte(0x00, 0b00000000);

	switch (whatChars) {
	case 0:
		learn_char_from_progmem(0, chr0);
		learn_char_from_progmem(1, chr1);
		learn_char_from_progmem(2, chr2);
		learn_char_from_progmem(3, chr3);
		learn_char_from_progmem(4, chr4);
		learn_char_from_progmem(5, chr5);
		learn_char_from_progmem(6, chr6);
		learn_char_from_progmem(7, chr7);
		break;
	case 8:
		learn_char_from_progmem(8, chr8);
		learn_char_from_progmem(9, chr9);
		learn_char_from_progmem(10, chr10);
		learn_char_from_progmem(11, chr11);
		learn_char_from_progmem(12, chr12);
		learn_char_from_progmem(13, chr13);
		learn_char_from_progmem(14, chr14);
		learn_char_from_progmem(15, chr15);
		break;
	case 16:
		learn_char_from_progmem(16, chr16);
		learn_char_from_progmem(17, chr17);
		learn_char_from_progmem(18, chr18);
		learn_char_from_progmem(19, chr19);
		learn_char_from_progmem(20, chr20);
		learn_char_from_progmem(21, chr21);
		learn_char_from_progmem(22, chr22);
		learn_char_from_progmem(23, chr23);
		break;
	case 24:
		learn_char_from_progmem(24, chr24);
		learn_char_from_progmem(25, chr25);
		learn_char_from_progmem(26, chr26);
		learn_char_from_progmem(27, chr27);
		learn_char_from_progmem(28, chr28);
		learn_char_from_progmem(29, chr29);
		learn_char_from_progmem(30, chr30);
		learn_char_from_progmem(31, chr31);
		break;
	case 32:
		learn_char_from_progmem(32, chr32);
		learn_char_from_progmem(33, chr33);
		learn_char_from_progmem(34, chr34);
		learn_char_from_progmem(35, chr35);
		learn_char_from_progmem(36, chr36);
		learn_char_from_progmem(37, chr37);
		learn_char_from_progmem(38, chr38);
		learn_char_from_progmem(39, chr39);
		break;
	case 40:
		learn_char_from_progmem(40, chr40);
		learn_char_from_progmem(41, chr41);
		learn_char_from_progmem(42, chr42);
		learn_char_from_progmem(43, chr43);
		learn_char_from_progmem(44, chr44);
		learn_char_from_progmem(45, chr45);
		learn_char_from_progmem(46, chr46);
		learn_char_from_progmem(47, chr47);
		break;
	case 48:
		learn_char_from_progmem(48, chr48);
		learn_char_from_progmem(49, chr49);
		learn_char_from_progmem(50, chr50);
		learn_char_from_progmem(51, chr51);
		learn_char_from_progmem(52, chr52);
		learn_char_from_progmem(53, chr53);
		learn_char_from_progmem(54, chr54);
		learn_char_from_progmem(55, chr55);
		break;
	case 56:
		learn_char_from_progmem(56, chr56);
		learn_char_from_progmem(57, chr57);
		learn_char_from_progmem(58, chr58);
		learn_char_from_progmem(59, chr59);
		learn_char_from_progmem(60, chr60);
		learn_char_from_progmem(61, chr61);
		learn_char_from_progmem(62, chr62);
		learn_char_from_progmem(63, chr63);
		break;
	case 64:
		learn_char_from_progmem(64, chr64);
		learn_char_from_progmem(65, chr65);
		learn_char_from_progmem(66, chr66);
		learn_char_from_progmem(67, chr67);
		learn_char_from_progmem(68, chr68);
		learn_char_from_progmem(69, chr69);
		learn_char_from_progmem(70, chr70);
		learn_char_from_progmem(71, chr71);
		break;
	case 72:
		learn_char_from_progmem(72, chr72);
		learn_char_from_progmem(73, chr73);
		learn_char_from_progmem(74, chr74);
		learn_char_from_progmem(75, chr75);
		learn_char_from_progmem(76, chr76);
		learn_char_from_progmem(77, chr77);
		learn_char_from_progmem(78, chr78);
		learn_char_from_progmem(79, chr79);
		break;
	case 80:
		learn_char_from_progmem(80, chr80);
		learn_char_from_progmem(81, chr81);
		learn_char_from_progmem(82, chr82);
		learn_char_from_progmem(83, chr83);
		learn_char_from_progmem(84, chr84);
		learn_char_from_progmem(85, chr85);
		learn_char_from_progmem(86, chr86);
		learn_char_from_progmem(87, chr87);
		break;
	case 88:
		learn_char_from_progmem(88, chr88);
		learn_char_from_progmem(89, chr89);
		learn_char_from_progmem(90, chr90);
		learn_char_from_progmem(91, chr91);
		learn_char_from_progmem(92, chr92);
		learn_char_from_progmem(93, chr93);
		learn_char_from_progmem(94, chr94);
		learn_char_from_progmem(95, chr95);
		break;
	case 96:
		learn_char_from_progmem(96, chr96);
		learn_char_from_progmem(97, chr97);
		learn_char_from_progmem(98, chr98);
		learn_char_from_progmem(99, chr99);
		learn_char_from_progmem(100, chr100);
		learn_char_from_progmem(101, chr101);
		learn_char_from_progmem(102, chr102);
		learn_char_from_progmem(103, chr103);
		break;
	case 104:
		learn_char_from_progmem(104, chr104);
		learn_char_from_progmem(105, chr105);
		learn_char_from_progmem(106, chr106);
		learn_char_from_progmem(107, chr107);
		learn_char_from_progmem(108, chr108);
		learn_char_from_progmem(109, chr109);
		learn_char_from_progmem(110, chr110);
		learn_char_from_progmem(111, chr111);
		break;
	case 112:
		learn_char_from_progmem(112, chr112);
		learn_char_from_progmem(113, chr113);
		learn_char_from_progmem(114, chr114);
		learn_char_from_progmem(115, chr115);
		learn_char_from_progmem(116, chr116);
		learn_char_from_progmem(117, chr117);
		learn_char_from_progmem(118, chr118);
		learn_char_from_progmem(119, chr119);
		break;
	case 120:
		learn_char_from_progmem(120, chr120);
		learn_char_from_progmem(121, chr121);
		learn_char_from_progmem(122, chr122);
		learn_char_from_progmem(123, chr123);
		learn_char_from_progmem(124, chr124);
		learn_char_from_progmem(125, chr125);
		learn_char_from_progmem(126, chr126);
		learn_char_from_progmem(127, chr127);
		break;
	case 128:
		learn_char_from_progmem(128, chr128);
		learn_char_from_progmem(129, chr129);
		learn_char_from_progmem(130, chr130);
		learn_char_from_progmem(131, chr131);
		learn_char_from_progmem(132, chr132);
		learn_char_from_progmem(133, chr133);
		learn_char_from_progmem(134, chr134);
		learn_char_from_progmem(135, chr135);
		break;
	case 136:
		learn_char_from_progmem(136, chr136);
		learn_char_from_progmem(137, chr137);
		learn_char_from_progmem(138, chr138);
		learn_char_from_progmem(139, chr139);
		learn_char_from_progmem(140, chr140);
		learn_char_from_progmem(141, chr141);
		learn_char_from_progmem(142, chr142);
		learn_char_from_progmem(143, chr143);
		break;
	case 144:
		learn_char_from_progmem(144, chr144);
		learn_char_from_progmem(145, chr145);
		learn_char_from_progmem(146, chr146);
		learn_char_from_progmem(147, chr147);
		learn_char_from_progmem(148, chr148);
		learn_char_from_progmem(149, chr149);
		learn_char_from_progmem(150, chr150);
		learn_char_from_progmem(151, chr151);
		break;
	case 152:
		learn_char_from_progmem(152, chr152);
		learn_char_from_progmem(153, chr153);
		learn_char_from_progmem(154, chr154);
		learn_char_from_progmem(155, chr155);
		learn_char_from_progmem(156, chr156);
		learn_char_from_progmem(157, chr157);
		learn_char_from_progmem(158, chr158);
		learn_char_from_progmem(159, chr159);
		break;
	case 160:
		learn_char_from_progmem(160, chr160);
		learn_char_from_progmem(161, chr161);
		learn_char_from_progmem(162, chr162);
		learn_char_from_progmem(163, chr163);
		learn_char_from_progmem(164, chr164);
		learn_char_from_progmem(165, chr165);
		learn_char_from_progmem(166, chr166);
		learn_char_from_progmem(167, chr167);
		break;
	case 168:
		learn_char_from_progmem(168, chr168);
		learn_char_from_progmem(169, chr169);
		learn_char_from_progmem(170, chr170);
		learn_char_from_progmem(171, chr171);
		learn_char_from_progmem(172, chr172);
		learn_char_from_progmem(173, chr173);
		learn_char_from_progmem(174, chr174);
		learn_char_from_progmem(175, chr175);
		break;
	case 176:
		learn_char_from_progmem(176, chr176);
		learn_char_from_progmem(177, chr177);
		learn_char_from_progmem(178, chr178);
		learn_char_from_progmem(179, chr179);
		learn_char_from_progmem(180, chr180);
		learn_char_from_progmem(181, chr181);
		learn_char_from_progmem(182, chr182);
		learn_char_from_progmem(183, chr183);
		break;
	case 184:
		learn_char_from_progmem(184, chr184);
		learn_char_from_progmem(185, chr185);
		learn_char_from_progmem(186, chr186);
		learn_char_from_progmem(187, chr187);
		learn_char_from_progmem(188, chr188);
		learn_char_from_progmem(189, chr189);
		learn_char_from_progmem(190, chr190);
		learn_char_from_progmem(191, chr191);
		break;
	case 192:
		learn_char_from_progmem(192, chr192);
		learn_char_from_progmem(193, chr193);
		learn_char_from_progmem(194, chr194);
		learn_char_from_progmem(195, chr195);
		learn_char_from_progmem(196, chr196);
		learn_char_from_progmem(197, chr197);
		learn_char_from_progmem(198, chr198);
		learn_char_from_progmem(199, chr199);
		break;
	case 200:
		learn_char_from_progmem(200, chr200);
		learn_char_from_progmem(201, chr201);
		learn_char_from_progmem(202, chr202);
		learn_char_from_progmem(203, chr203);
		learn_char_from_progmem(204, chr204);
		learn_char_from_progmem(205, chr205);
		learn_char_from_progmem(206, chr206);
		learn_char_from_progmem(207, chr207);
		break;
	case 208:
		learn_char_from_progmem(208, chr208);
		learn_char_from_progmem(209, chr209);
		learn_char_from_progmem(210, chr210);
		learn_char_from_progmem(211, chr211);
		learn_char_from_progmem(212, chr212);
		learn_char_from_progmem(213, chr213);
		learn_char_from_progmem(214, chr214);
		learn_char_from_progmem(215, chr215);
		break;
	case 216:
		learn_char_from_progmem(216, chr216);
		learn_char_from_progmem(217, chr217);
		learn_char_from_progmem(218, chr218);
		learn_char_from_progmem(219, chr219);
		learn_char_from_progmem(220, chr220);
		learn_char_from_progmem(221, chr221);
		learn_char_from_progmem(222, chr222);
		learn_char_from_progmem(223, chr223);
		break;
	case 224:
		learn_char_from_progmem(224, chr224);
		learn_char_from_progmem(225, chr225);
		learn_char_from_progmem(226, chr226);
		learn_char_from_progmem(227, chr227);
		learn_char_from_progmem(228, chr228);
		learn_char_from_progmem(229, chr229);
		learn_char_from_progmem(230, chr230);
		learn_char_from_progmem(231, chr231);
		break;
	case 232:
		learn_char_from_progmem(232, chr232);
		learn_char_from_progmem(233, chr233);
		learn_char_from_progmem(234, chr234);
		learn_char_from_progmem(235, chr235);
		learn_char_from_progmem(236, chr236);
		learn_char_from_progmem(237, chr237);
		learn_char_from_progmem(238, chr238);
		learn_char_from_progmem(239, chr239);
		break;
	case 240:
		learn_char_from_progmem(240, chr240);
		learn_char_from_progmem(241, chr241);
		learn_char_from_progmem(242, chr242);
		learn_char_from_progmem(243, chr243);
		learn_char_from_progmem(244, chr244);
		learn_char_from_progmem(245, chr245);
		learn_char_from_progmem(246, chr246);
		learn_char_from_progmem(247, chr247);
		break;
	case 248:
		learn_char_from_progmem(248, chr248);
		learn_char_from_progmem(249, chr249);
		learn_char_from_progmem(250, chr250);
		learn_char_from_progmem(251, chr251);
		learn_char_from_progmem(252, chr252);
		learn_char_from_progmem(253, chr253);
		learn_char_from_progmem(254, chr254);
		learn_char_from_progmem(255, chr255);
		break;
	default:
		//write none;
		break;
	}
}

void learn_all_smart() {
	for (uint8_t i = 0; i < 32; i++) {
		learnCharacters(i * 8);
	}
}

#endif //WRITECHARS
