/*****************************
 * ECE 4760: Final Project   *
 * Human Tetris              *
 * Video Arcade Edition      *
 *                           *
 * Adam Papamarcos (aip23)   *
 * Kerran Flanagan (kaf42)   *
 * Cornell University        *
 * Spring 2010               *
 *****************************/

/****************************************************************************
 *   Copyright (C) 2009 by Claas Anders "CaScAdE" Rathje                    *
 *   admiralcascade@gmail.com                                               *
 *   Project-URL: http://www.mylifesucks.de/oss/c-osd/                      *
 *                                                                          *
 *   This program is free software; you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by   *
 *   the Free Software Foundation; either version 2 of the License.         *
 *                                                                          *
 *   This program is distributed in the hope that it will be useful,        *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for more details.                           *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to the                          *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 *                                                                          *
 *   Extended and modified for ATmega644 in 2010 by                         *
 *   Kerran Flanagan and Adam Papamarcos.                                   *
 ****************************************************************************/

#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h> 
#include "max7456_spi.h"
#include "macros.h"
#include "max7456_learn.h"

//uart debugging
#include <avr/io.h>
#include <stdio.h>
#include "uart.h"

/* ##########################################################################
 * MAX7456 SPI & Display stuff
 * ##########################################################################*/

/**
 * Send a byte through SPI
 */
void spi_send(uint8_t byte) {

	//mega644 spi controller
	/* Start transmission */
	SPDR = byte;
	/* Wait for transmission complete */
	while (!(SPSR & (1 << SPIF)))
		;
	//junk = SPDR; //get back junk

	//printf("sent %X\n\r", byte);
}

unsigned char spi_read() {
	unsigned char data = 0;

	//printf("starting read\n\r");

	//mega644 spi controller

	//SPDR = junk; //send junk

	/* Wait for reception complete */
	while (!(SPSR & (1 << SPIF))) {
		//printf("reading...\n\r");
	}
	/* Return Data Register */
	data = SPDR;

	//printf("read %X\n\r", data);
	return data; // Finally return the read data
}

/**
 *  Send <byte> to <address> of MAX7456
 */
void spi_send_byte(uint8_t address, uint8_t byte) {
	// start sending
	MAX_CS_LOW

#if (UART_DEBUG == 1)
	printf("send addr = %X data = %X\n\r",address, byte);
#endif
	spi_send(address);
	spi_send(byte);

	// end sending
	MAX_CS_HIGH
}

uint8_t spi_read_byte(uint8_t address) {
	//start
	MAX_CS_LOW

	spi_send(address);
	unsigned char data = spi_read();

	//end
	MAX_CS_HIGH

	//printf("read addr = %X data = %X\n\r", address, data);

	return data;
}

/**
 *  write a <character> to <address> of MAX7456 display memory
 */
void write_char(uint16_t address, char character) {
	spi_send_byte(0x05, (address & 0xFF00) >> 8); // DMAH
	spi_send_byte(0x06, (address & 0x00FF)); // DMAL
	spi_send_byte(0x07, character); // DMDI
}

/**
 *  write a character <attribute> to <address> of MAX7456 display memory
 */
void write_char_att(uint16_t address, char attribute) {
	// the only important part is that the DMAH[1] is set
	// so we add 2 which binary is the 2nd lowest byte
	spi_send_byte(0x05, ((address & 0xFF00) >> 8) | 2); // DMAH
	spi_send_byte(0x06, (address & 0x00FF)); // DMAL
	spi_send_byte(0x07, attribute); // DMDI
}

/**
 *  write a <character> at <x>/<y> to MAX7456 display memory
 */
void write_char_xy(uint8_t x, uint8_t y, char character) {
	uint16_t address = y * 30 + x;
	write_char(address, character);
}

/**
 *  write a  character <attribute> at <x>/<y> to MAX7456 display memory
 */
void write_char_att_xy(uint8_t x, uint8_t y, char attribute) {
	uint16_t address = y * 30 + x;
	write_char_att(address, attribute);
}

/**
 *  clear display by writing blank characters all over it
 */
void clear(void) {
	uint16_t memory_address = 0;
	for (unsigned int a = 0; a < 480; a++) {
		write_char(memory_address++, 0);
	}
}

/**
 * remap an ascii char to its location in the MAX7456 character memory
 */
char remap_ascii_char(char c) {

	if (c > 48 && c <= 57)
		c -= 48; // remap numbers
	else if (c >= 65 && c <= 90)
		c -= 54; // remap big letters
	else if (c >= 97 && c <= 122)
		c -= 60; // remap small letters
	else
		switch (c) {
		case 32:
			c = 0; // remap space
			break;
		case '0':
			c = 10; // remap zero
			break;
		case '(':
			c = 63; // remap
			break;
		case ')':
			c = 64; // remap
			break;
		case '.':
			c = 65; // remap
			break;
		case '?':
			c = 66; // remap
			break;
		case ';':
			c = 67; // remap
			break;
		case ':':
			c = 68; // remap
			break;
		case ',':
			c = 69; // remap
			break;
		case '\'':
			c = 70; // remap
			break;
		case '/':
			c = 71; // remap
			break;
		case '"':
			c = 72; // remap
			break;
		case '-':
			c = 73; // remap minus
			break;
		case '<':
			c = 74; // remap
			break;
		case '>':
			c = 75; // remap
			break;
		case '@':
			c = 76; // remap
			break;
		case '!':
			c = 77; //remap
			break;
		}
	return c;
}

/**
 *  write an ascii <character> to <address> of MAX7456 display memory
 */
void write_ascii_char(uint16_t address, char c) {
	c = remap_ascii_char(c);
	write_char(address, c);
}

/**
 *  write an ascii <string> at <x>/<y> to MAX7456 display memory
 */
void write_ascii_string(uint8_t x, uint8_t y, char *string) {
	while (*string) {
		write_ascii_char(((x++) + (y * 30)), *string);
		string++;
	}
}



/**
 * for testing write all chars to screen
 */
void write_all_chars() {
	// Setup Video Mode: NTSC + enable display immediately (VM0)
		spi_send_byte(0x00, 0b00001000);
		// 8bit mode
			spi_send_byte(0x04, 0b01000000);


	uint16_t x = 3, y = 1, t = 0;
	while (t < 256) {
		write_char_xy(x++, y, t++);
		if (x > 25) {
			y++;
			x = 3;
		}
	}
}

/** Setup SPI for talking to OSD. */
void init_SPI() {

	//set up i/o data direction
	//DDRB.4 = 1; //output chip select
	//DDRB.5 = 1; //output MOSI
	//DDRB.6 = 0; //input MISO
	//DDRB.7 = 1; //output SCLK
	//DDRB = (1 << DDB4) | (1 << DDB5) | (1 << DDB7);
	SET(DDRB,DDB4);
	SET(DDRB,DDB5);
	SET(DDRB,DDB7);
	CLR(DDRB,DDB6);

	// initialize chip select high
	SET(PORTB,DDB4);

	//set up SPI
	//bit 7 SPIE=0 no ISR
	//bit 6 SPE=1 enable spi
	//bit 5 DORD=0 msb first
	//bit 4 MSTR=1 Mega644 is spi master
	//bit 3 CPLO=0 clock polarity
	//bit 2 CPHA=0 clock phase
	//bit 1,0 rate sel 0 << SPR1, 0 << SPR0 sets sclk freq to CLK/4 MHz (with SPSR = 0 << SPI2X)
	SET(SPCR,SPE);
	SET(SPCR,MSTR);
	CLR(SPCR,CPOL);
	CLR(SPCR,CPHA);
	CLR(SPCR,SPR1);
	CLR(SPCR,SPR0);
	CLR(SPSR,SPI2X);

	// reset the MAX7456 to be sure any undefined states do no harm
	MAX_RESET_LOW
	_delay_ms(100); //spec is 50ms
	MAX_RESET_HIGH

	// give the maxim time to come up
	// takes 50ms for OSD to stabilize when turned on, stat[6] = 0 when rdy
	//_delay_ms(50);
	unsigned char stat = spi_read_byte(0xA);
	//printf("status reg = %X\n\r", stat);
	while (stat & 0b01000000) {
		stat = spi_read_byte(0xA);
		//printf("status reg = %X\n\r", stat);
	}

	// Setup Video Mode: NTSC + enable display immediately (VM0)
	spi_send_byte(0x00, 0b00001000);

	// clear all display-mem (DMM)
	spi_send_byte(0x04, 0b00000100);

	// clearing takes 20uS according to maxim so lets wait longer
	_delay_us(60);

	// 8bit mode
	spi_send_byte(0x04, 0b01000000);

	// write blank chars to whole screen
	clear();

	// enable automatic OSD black level control
	unsigned char osdbl = spi_read_byte(0xEC);
	spi_send_byte(0x6C, osdbl & 0b11101111);

}

